import os
import uuid
import datetime
from django.conf import settings
from redactor.utils import import_class


class BaseUploaderRedactor(object):
    """
    Base class for uploader handler.
    """

    def __init__(self, upload_file, upload_to=None):
        self.upload_file = upload_file
        self.upload_to = upload_to

        file_storage_class = getattr(settings, 'REDACTOR_FILE_STORAGE',
                                     'django.core.files.storage.DefaultStorage')

        # File storage can either be a Storage instance (currently deprecated),
        # or a class which we should instantiate ourselves
        file_storage = import_class(file_storage_class)
        if isinstance(file_storage, type):
            # The class case
            file_storage = file_storage()
        self.file_storage = file_storage

    def get_file(self):
        """
        Return file object
        """
        return self.upload_file

    def get_full_path(self):
        """
        Return full path for file:
        /REDACTOR_UPLOAD/filename.etc
        """
        return os.path.join(self.get_upload_path(), self.get_filename())

    def save_file(self):
        """
        Save file and return real path
        """
        if not hasattr(self, 'real_path'):
            self.real_path = self.file_storage.save(self.get_full_path(),
                                                    self.get_file())
        return self.real_path

    def get_url(self):
        """
        Return url for file if he saved else None
        """
        if not hasattr(self, 'real_path'):
            return None
        else:
            return self.file_storage.url(self.real_path)

    def get_filename(self):
        """
        Should return the file name
        """
        raise NotImplementedError

    def get_upload_path(self):
        """
        Should return the directory for file storage
        """
        raise NotImplementedError

    @staticmethod
    def get_default_upload_path():
        return getattr(settings, 'REDACTOR_UPLOAD', 'redactor/')


class SimpleUploader(BaseUploaderRedactor):
    """
    Standard uploader: default directory, default name
    """
    def get_filename(self):
        return self.upload_file.name

    def get_upload_path(self):
        return self.upload_to or self.get_default_upload_path()


class UUIDUploader(SimpleUploader):
    """
    Handler that renames files based on UUID

    /REDACTOR_UPLOAD/546de5b5-cf05-4b47-9379-3f964732b802.etc
    """
    def get_filename(self):
        if not hasattr(self, 'filename'):
            # save filename prevents the generation of a new
            extension = self.upload_file.name.split('.')[-1]
            self.filename = '{0}.{1}'.format(uuid.uuid4(), extension)
        return self.filename


class DateDirectoryUploader(SimpleUploader):
    """
    Handler  that saves files in a directory based on the current date

    /2014/3/28/filename.etc
    """
    def get_upload_path(self):
        today = datetime.datetime.today()
        path = '{0}/{1}/{2}'.format(today.year, today.month, today.day)
        return path




from django.conf.urls import url

from redactor.views import RedactorUploadView
from redactor.forms import FileForm, ImageForm


urlpatterns = [
    url(r'^upload/image/(?P<upload_to>.*)',
        RedactorUploadView.as_view(form_class=ImageForm),
        name='redactor_upload_image'),

    url(r'^upload/file/(?P<upload_to>.*)',
        RedactorUploadView.as_view(form_class=FileForm),
        name='redactor_upload_file'),
]



import json
from django.conf import settings
from django.utils.translation import ugettext as _
from django.http import HttpResponse
from django.views.generic import FormView
from django.contrib.admin.views.decorators import staff_member_required
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.utils.encoding import force_str

from redactor.forms import ImageForm
from redactor.utils import import_class, is_module_image_installed

# deal with python3 basestring
try:
    unicode = unicode
except NameError:
    basestring = (str, bytes)
else:
    basestring = basestring


class RedactorUploadView(FormView):
    form_class = ImageForm
    http_method_names = ('post',)
    upload_to = getattr(settings, 'REDACTOR_UPLOAD', 'redactor/')
    upload_handler = getattr(settings, 'REDACTOR_UPLOAD_HANDLER',
                             'redactor.handlers.SimpleUploader')
    auth_decorator = getattr(settings, 'REDACTOR_AUTH_DECORATOR',
                             staff_member_required)
    if isinstance(auth_decorator, basestring):
        # Given decorator is string, probably because user can't import eg.
        # django.contrib.auth.decorators.login_required in settings level.
        # We are expected to import it on our own.
        auth_decorator = import_class(auth_decorator)

    @method_decorator(csrf_exempt)
    @method_decorator(auth_decorator)
    def dispatch(self, request, *args, **kwargs):
        if not is_module_image_installed():
            data = {
                'error': True,
                'message': _("ImproperlyConfigured: Neither Pillow nor PIL could be imported: No module named 'Image'")
            }
            return HttpResponse(json.dumps(data), status=400,
                                content_type='application/json')
        return super(RedactorUploadView, self).dispatch(request,
                                                        *args, **kwargs)

    def form_invalid(self, form):
        # TODO: Needs better error messages
        try:
            message = form.errors.values()[-1][-1]
        except:
            message = _('Invalid file.')
        data = {'error': True, 'message': message}
        return HttpResponse(json.dumps(data), status=400,
                            content_type='application/json')

    def form_valid(self, form):
        file_ = form.cleaned_data['file']
        handler_class = import_class(self.upload_handler)
        uploader = handler_class(file_,
                                 upload_to=self.kwargs.get('upload_to', None))
        uploader.save_file()
        file_name = force_str(uploader.get_filename())
        file_url = force_str(uploader.get_url())
        file_name_key = 'id' if self.form_class is ImageForm else 'name'
        data = {'url': file_url, file_name_key: file_name}
        return HttpResponse(json.dumps(data), content_type='application/json')



from django.core.exceptions import ImproperlyConfigured
from importlib import import_module

try:
    from django.utils.encoding import force_text
except ImportError:
    from django.utils.encoding import force_unicode as force_text
from django.utils.functional import Promise

import json


def import_class(path):
    path_bits = path.split('.')

    if len(path_bits) < 2:
        message = "'{0}' is not a complete Python path.".format(path)
        raise ImproperlyConfigured(message)

    class_name = path_bits.pop()
    module_path = '.'.join(path_bits)
    module_itself = import_module(module_path)

    if not hasattr(module_itself, class_name):
        message = "The Python module '{0}' has no '{1}' class.".format(
            module_path,
            class_name
        )
        raise ImportError(message)

    return getattr(module_itself, class_name)


def is_module_image_installed():
    try:
        from PIL import Image
        from PIL import ImageFile
    except ImportError:
        try:
            import Image
            import ImageFile
        except ImportError:
            return False
    return True


class LazyEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, Promise):
            return force_text(obj)
        return super(LazyEncoder, self).default(obj)


def json_dumps(data):
    return json.dumps(data, cls=LazyEncoder)



from django import forms
from django.forms import widgets
from django.utils.safestring import mark_safe
from django.urls import reverse_lazy
from django.conf import settings

from redactor.utils import json_dumps


class RedactorEditor(widgets.Textarea):
    def __init__(self, *args, **kwargs):
        upload_to = kwargs.pop('upload_to', '')
        self.options = dict(getattr(settings, 'REDACTOR_OPTIONS', {}))
        self.options.update(kwargs.pop('redactor_options', {}))

        if kwargs.pop('allow_file_upload', True):
            self.options['fileUpload'] = reverse_lazy(
                'redactor_upload_file', kwargs={'upload_to': upload_to}
            )
        if kwargs.pop('allow_image_upload', True):
            self.options['imageUpload'] = reverse_lazy(
                'redactor_upload_image', kwargs={'upload_to': upload_to}
            )

        widget_attrs = {'class': 'redactor-box'}
        widget_attrs.update(kwargs.get('attrs', {}))
        widget_attrs.update({'data-redactor-options': self.options})

        kwargs['attrs'] = widget_attrs
        super(RedactorEditor, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None, renderer=None):
        """
        Must parse self.options with json_dumps on self.render.
        Because at some point Django calls RedactorEditor.__init__ before
        loading the urls, and it will break.
        """
        attrs['data-redactor-options'] = json_dumps(self.options)
        html = super(RedactorEditor, self).render(name, value, attrs, renderer=renderer)
        return mark_safe(html)

    def _media(self):
        _min = '' if settings.DEBUG else '.min'
        js = (
            'redactor/jquery.redactor.init.js',
            'redactor/redactor_{}.js'.format(_min),
            'redactor/langs/{}.js'.format(self.options.get('lang', 'en')),
        )

        if 'plugins' in self.options:
            plugins = self.options.get('plugins')
            for plugin in plugins:
                js = js + (
                    'redactor/plugins/{}.js'.format(plugin),
                )

        css = {
            'all': (
                'redactor/redactor{}.css'.format(_min),
                'redactor/django_admin.css',
            )
        }
        return forms.Media(css=css, js=js)
    media = property(_media)



from django.db.models import TextField
from django.conf import settings
from django.contrib.admin import widgets as admin_widgets
from redactor.widgets import RedactorEditor


class RedactorField(TextField):
    def __init__(self, *args, **kwargs):
        options = kwargs.pop('redactor_options', {})
        upload_to = kwargs.pop('upload_to', '')
        allow_file_upload = kwargs.pop('allow_file_upload', False)
        allow_image_upload = kwargs.pop('allow_image_upload', True)
        self.widget = RedactorEditor(
            redactor_options=options,
            upload_to=upload_to,
            allow_file_upload=allow_file_upload,
            allow_image_upload=allow_image_upload
        )
        super(RedactorField, self).__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {'widget': self.widget}
        defaults.update(kwargs)

        if defaults['widget'] == admin_widgets.AdminTextareaWidget:
            defaults['widget'] = self.widget
        return super(RedactorField, self).formfield(**defaults)


if 'south' in settings.INSTALLED_APPS:
    from south.modelsinspector import add_introspection_rules
    add_introspection_rules([], ["^redactor\.fields\.RedactorField"])
